unit modul_pio;

{$mode objfpc}

interface

uses
  Classes, SysUtils, CRT;



const


  // PIO OUT
  PIO_OUT_CONOUT           = $001;
  PIO_OUT_RW_VECTOR_LO     = $002;
  PIO_OUT_READ_VECTOR_HI   = $003;
  PIO_OUT_WRITE_VECTOR_HI  = $004;
  PIO_OUT_LIST             = $005;
  PIO_OUT_LOAD_BDOS        = $006;
  PIO_OUT_LOAD_CCP         = $007;

  PIO_OUT_HALT             = $0FF;


  // PIO IN
  PIO_IN_CONST             = $001;
  PIO_IN_CONIN             = $002;





// DRV_VEC

OFFSET_SECTOR = 0;
OFFSET_TRACK  = 1;
OFFSET_DISKNO = 3;
OFFSET_DMAAD  = 4;







type


  Tmod_pio = class(TObject)

    constructor Create; overload;
    destructor Destroy; override;

  private
    RW_VECTOR_LO : Byte;
    function CONST_auslesen : Byte;
    function CONIN_auslesen : Byte;

  public
    CONIN_Buffer : String;

    function Input_PIO(Port : Byte) : Byte; { ein Byte von der PIO lesen }
    procedure Output_PIO(Port, Data : Byte); { ein Byte in die PIO schreiben }

  end; // Tmod_pio = class(TObject)




var
  mod_pio: Tmod_pio;



implementation


uses
  modul_image, modul_ram, Emulator;



constructor Tmod_pio.Create;
begin
  inherited Create;

  CONIN_Buffer:='';

end; { constructor Tmod_pio.Create }








function Tmod_pio.CONST_auslesen : Byte;
var
  Ergebnis : Byte;



begin
  Ergebnis:=0;

  if Length(CONIN_Buffer)>0 then begin

    Ergebnis:=$FF;

  end { if Length(CONIN_Buffer)>0 then }

  else begin

    if KeyPressed then begin
      Ergebnis:=$FF;
    end;

  end;

  Result := Ergebnis;

end; { function Tmod_pio.CONST_auslesen }





function Tmod_pio.CONIN_auslesen : Byte;
var
  Ergebnis : Byte;
  Zeichen : Char;

begin
  Ergebnis:=0;

  if Length(CONIN_Buffer)>0 then begin

    Ergebnis:=Byte(CONIN_Buffer[1]);
    Delete(CONIN_Buffer,1,1);

  end { if Length(CONIN_Buffer)>0 then }
  else begin
    if KeyPressed then begin
      Zeichen:=ReadKey;
      Ergebnis:=ORd(Zeichen);

      if Ergebnis=0 then begin

        Zeichen:=ReadKey;
        Ergebnis:=ORd(Zeichen);

      end; { if Ergebnis=0 then }


    end; { if KeyPressed then }
  end; { else Length(CONIN_Buffer)>0 }


  Result := Ergebnis;

end; { function Tmod_pio.CONIN_auslesen }






function Tmod_pio.Input_PIO(Port : Byte) : Byte; { ein Byte von der PIO lesen }
var
  Ergebnis : Byte;
begin
  Ergebnis := 0;

  {
  PIO_IN_CONST		EQU	001H
  PIO_IN_CONIN		EQU	002H
  }

  case Port of

    PIO_IN_CONST             : begin
                                 Ergebnis:=CONST_auslesen;
                               end; { PIO_OUT_CONOUT }

    PIO_IN_CONIN             : begin
                                 Ergebnis:=CONIN_auslesen;
                               end; { PIO_OUT_RW_VECTOR_LO }

  else


  end; { case Adress of }

  Result:=Ergebnis;

end; { function Tmod_pio.Read_Byte }





procedure Tmod_pio.Output_PIO(Port, Data : Byte); { ein Byte in die PIO schreiben }
var
  RW_VECTOR_HI, Byte_Low, Byte_High : Byte;
  RW_VECTOR, VAR_DMAAD : Word;
  VEC_Zaehler : Integer;
begin


  {
  PIO_OUT_CONOUT           : Byte = $001;
  PIO_OUT_RW_VECTOR_LO     : Byte = $002;
  PIO_OUT_READ_VECTOR_HI   : Byte = $003;
  PIO_OUT_WRITE_VECTOR_HI  : Byte = $004;
  PIO_OUT_HALT
  }

  case Port of

    PIO_OUT_CONOUT           : begin
                                 Write(CHR(Data));

                               end; { PIO_OUT_CONOUT }

    PIO_OUT_RW_VECTOR_LO     : begin
                                 RW_VECTOR_LO:=Data;
                               end; { PIO_OUT_RW_VECTOR_LO }


    PIO_OUT_READ_VECTOR_HI   : begin
                                 RW_VECTOR_HI:=Data;
                                 RW_VECTOR := (RW_VECTOR_HI shl 8) + RW_VECTOR_LO;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 mod_image.VAR_SECTOR := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 mod_image.VAR_TRACK := (Byte_High shl 8) + Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 mod_image.VAR_DISKNO := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 VAR_DMAAD := (Byte_High shl 8) + Byte_Low;

                                 for VEC_Zaehler:=0 to 3 do begin

                                   mod_image.Read_IMAGE_Sektor(VEC_Zaehler, VAR_DMAAD);

                                 end;

                               end; { PIO_OUT_READ_VECTOR_HI }



    PIO_OUT_WRITE_VECTOR_HI  : begin
                                 RW_VECTOR_HI:=Data;
                                 RW_VECTOR := (RW_VECTOR_HI shl 8) + RW_VECTOR_LO;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 mod_image.VAR_SECTOR := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 mod_image.VAR_TRACK := (Byte_High shl 8) + Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 mod_image.VAR_DISKNO := Byte_Low;

                                 Byte_Low := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 Byte_High := mod_ram.Read_Byte(RW_VECTOR);
                                 INC(RW_VECTOR);
                                 VAR_DMAAD := (Byte_High shl 8) + Byte_Low;

                                 for VEC_Zaehler:=0 to 3 do begin

                                   mod_image.Write_IMAGE_Sektor(VEC_Zaehler, VAR_DMAAD);

                                 end;

                               end; { PIO_OUT_WRITE_VECTOR_HI }


    PIO_OUT_LIST             : begin
                                 // erst mal nur Dummy

                               end; { PIO_OUT_LIST }



    PIO_OUT_LOAD_BDOS        : begin
                                 mod_image.Lade_BDOS_in_RAM(Data);
                               end; { PIO_OUT_LOAD_BDOS }


    PIO_OUT_LOAD_CCP         : begin
                                 mod_image.Lade_CCP_in_RAM(Data);
                               end; { PIO_OUT_LOAD_CCP }


    PIO_OUT_HALT             : begin
                                 Z80.Schleife_aktiv:=false;
                               end; { PIO_OUT_HALT }

  else

  end; { case Adress of }

end; { procedure Tmod_pio.Write_Byte }





destructor Tmod_pio.Destroy;
begin

  inherited Destroy;
end; { destructor Tmod_pio.Destroy }



end.

         
